package com.fs.store.service.impl;

import java.time.LocalDateTime;
import java.util.*;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.fs.common.core.domain.R;
import com.fs.common.utils.DateUtils;
import com.fs.store.domain.FsCouponSchedule;
import com.fs.store.domain.FsCouponScheduleLog;
import com.fs.store.enums.IcgProcessStatusEnum;
import com.fs.store.enums.IcgScheduleOperationTypeEnum;
import com.fs.store.mapper.FsCouponScheduleLogMapper;
import com.fs.store.mapper.FsCouponScheduleMapper;
import com.fs.store.param.FsStoreCouponReceiveParam;
import com.fs.store.service.IFsCouponScheduleService;
import com.fs.store.service.IFsStoreCouponIssueService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

/**
 * 定时发放优惠券队列Service业务层处理
 *
 * @author fs
 * @date 2025-03-08
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class FsCouponScheduleServiceImpl implements IFsCouponScheduleService
{
    private final FsCouponScheduleMapper fsCouponScheduleMapper;
    private final IFsStoreCouponIssueService fsStoreCouponIssueService;
    private final FsCouponScheduleLogMapper fsCouponScheduleLogMapper;
    /**
     * 查询定时发放优惠券队列
     *
     * @param id 定时发放优惠券队列ID
     * @return 定时发放优惠券队列
     */
    @Override
    public FsCouponSchedule selectFsCouponScheduleById(Long id)
    {
        return fsCouponScheduleMapper.selectFsCouponScheduleById(id);
    }

    @Override
    public FsCouponSchedule selectFsCouponScheduleByOrderId(String orderId) {
        return fsCouponScheduleMapper.selectFsCouponScheduleByOrderId(orderId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class,propagation = Propagation.REQUIRED)
    public void setScheduleCouponRefund(String orderCode) {
        FsCouponSchedule fsCouponSchedule = fsCouponScheduleMapper.selectFsCouponScheduleByOrderId(orderCode);
        if(ObjectUtil.isNotNull(fsCouponSchedule)){
            fsCouponSchedule.setStatus(IcgProcessStatusEnum.REJECTED_OR_RETURNED.getCode());
            fsCouponScheduleMapper.updateFsCouponSchedule(fsCouponSchedule);
        }
    }

    /**
     * 查询定时发放优惠券队列列表
     *
     * @param fsCouponSchedule 定时发放优惠券队列
     * @return 定时发放优惠券队列
     */
    @Override
    public List<FsCouponSchedule> selectFsCouponScheduleList(FsCouponSchedule fsCouponSchedule)
    {
        return fsCouponScheduleMapper.selectFsCouponScheduleList(fsCouponSchedule);
    }

    /**
     * 新增定时发放优惠券队列
     *
     * @param fsCouponSchedule 定时发放优惠券队列
     * @return 结果
     */
    @Override
    public int insertFsCouponSchedule(FsCouponSchedule fsCouponSchedule)
    {
        fsCouponSchedule.setCreateTime(DateUtils.getNowDate());
        return fsCouponScheduleMapper.insertFsCouponSchedule(fsCouponSchedule);
    }

    /**
     * 修改定时发放优惠券队列
     *
     * @param fsCouponSchedule 定时发放优惠券队列
     * @return 结果
     */
    @Override
    public int updateFsCouponSchedule(FsCouponSchedule fsCouponSchedule)
    {
        fsCouponSchedule.setUpdateTime(DateUtils.getNowDate());
        return fsCouponScheduleMapper.updateFsCouponSchedule(fsCouponSchedule);
    }

    /**
     * 批量删除定时发放优惠券队列
     *
     * @param ids 需要删除的定时发放优惠券队列ID
     * @return 结果
     */
    @Override
    public int deleteFsCouponScheduleByIds(Long[] ids)
    {
        return fsCouponScheduleMapper.deleteFsCouponScheduleByIds(ids);
    }

    /**
     * 删除定时发放优惠券队列信息
     *
     * @param id 定时发放优惠券队列ID
     * @return 结果
     */
    @Override
    public int deleteFsCouponScheduleById(Long id)
    {
        return fsCouponScheduleMapper.deleteFsCouponScheduleById(id);
    }

    /**
     * 发放优惠券
     */
    @Override
    public void issueCoupon() {
        List<FsCouponSchedule> fsCouponSchedules = fsCouponScheduleMapper.selectPendingCouponList();
        List<FsCouponScheduleLog> logList = new ArrayList<>();
        for (FsCouponSchedule fsCouponSchedule : fsCouponSchedules) {

            FsCouponScheduleLog log = new FsCouponScheduleLog();

            log.setScheduleId(fsCouponSchedule.getId());
            log.setOrderId(fsCouponSchedule.getOrderId());
            log.setUserId(fsCouponSchedule.getUserId());
            log.setStatusBefore(fsCouponSchedule.getStatus());
            log.setSendTimeBefore(fsCouponSchedule.getSendTime());
            log.setRetryCountBefore(fsCouponSchedule.getRetryCount());
            log.setOperator("SYSTEM");
            log.setOperationTime(LocalDateTime.now());

            FsStoreCouponReceiveParam param = new FsStoreCouponReceiveParam();
            Long userId = fsCouponSchedule.getUserId();
            // 领取优惠券
            R receive = fsStoreCouponIssueService.receive(String.valueOf(userId), param);
            Set<String> operationTypeSet = new HashSet<>();
            if(fsCouponSchedule.getRetryCount()>0) {
                operationTypeSet.add(IcgScheduleOperationTypeEnum.RETRY.getCode());
            }
            // 如果不等于200表示失败了
            if(ObjectUtil.equal(200,receive.get("code"))){
                fsCouponSchedule.setErrorMessage(JSON.toJSONString(receive));
                fsCouponSchedule.setStatus(IcgProcessStatusEnum.FAILED.getCode());
                fsCouponSchedule.setRetryCount(fsCouponSchedule.getRetryCount()+1);

                log.setStatusAfter(IcgProcessStatusEnum.FAILED.getCode());
                log.setErrorMessage(fsCouponSchedule.getErrorMessage());

                operationTypeSet.add(IcgScheduleOperationTypeEnum.ERROR.getCode());
            } else {
                fsCouponSchedule.setStatus(IcgProcessStatusEnum.SUCCESS.getCode());
                fsCouponSchedule.setActualSendTime(LocalDateTime.now());
                fsCouponSchedule.setSendTime(fsCouponSchedule.getActualSendTime().plusMonths(1));
                fsCouponSchedule.setCount(fsCouponSchedule.getCount()+1);

                log.setStatusAfter(IcgProcessStatusEnum.SUCCESS.getCode());
                log.setActualSendTime(fsCouponSchedule.getActualSendTime());
                log.setSendTimeAfter(fsCouponSchedule.getSendTime());
                operationTypeSet.add(IcgScheduleOperationTypeEnum.NORMAL.getCode());
            }
            fsCouponSchedule.setUpdateTime(new Date());

            log.setOperationType(String.join(",",operationTypeSet));
            logList.add(log);
        }
        // 批量更新发放优惠券队列表
        fsCouponScheduleMapper.updateBatchFsCouponSchedule(fsCouponSchedules);
        // 批量添加执行日志
        fsCouponScheduleLogMapper.batchInsert(logList);
        // 批量进行微信公众号消息通知

    }
}
